#pragma once

#include <string>
#include <vector>

class IScenarioLoader;
class ScenarioData;
class IObjectiveFunction;
class IVehicleSolver;
class ISystemSolver;
class ElectricCircuit;
class IArrangement;

class IScenario
{
protected:
	std::vector<IObjectiveFunction*> mObjectiveFunction;
	std::vector<IVehicleSolver*> mVehicleSolver;
	std::vector<ISystemSolver*> mSystemSolver;
	std::vector<IArrangement*> mArrangement;
	std::vector<ElectricCircuit*> mSolverCircuit;
	IScenarioLoader* mRefScenarioLoader;

	unsigned int mDataIndexSimulator = 0;

public:
	IScenario(IScenarioLoader* inScenarioLoader):mRefScenarioLoader(inScenarioLoader){}
	virtual ~IScenario();

	virtual ScenarioData* GetRefData() const = 0;

	//Return if a vehicle was solved
	virtual bool SolveNextVehicle();
	virtual void SolveAllVehicle();
	virtual void SolveAllSystem();

	virtual ISystemSolver* const GetSystemSolver(int i) const {
		return mSystemSolver[i];
	}

	virtual void SetActivation(bool toActivate[]);

	virtual void LoadNamelessVehicle(std::istream& inDataVehicle);

	virtual std::string toString() const;
	std::string toStringCost() const;
	virtual std::string toStringPerformance() const;
	virtual std::string toStringInteractive() const ;
};