#include "Path.h"
#include <sstream>
#include <iomanip>
#include <vector>
#include <iomanip>

#include "ElectricTerminal.h"
//TEMP to remove
#include "Action.h"
#include "ActionRoad.h"
#include "ActionTerminal.h"
#include "ActionWait.h"

void Path::recomputeDistance()
{
	mDistance = 0.0f;
	for (auto it = mSequenceAction.begin(); it != mSequenceAction.end(); ++it)
	{
		if ((*it)->getType() == Action::eRoad)
		{
			ActionRoad* ar = dynamic_cast<ActionRoad*>(*it);
			mDistance += ar->mDistance;
		}
	}
}

Path::Path(Point a, Point b, double autonomie)
{
	mStart = a;
	mEnd = b;
	mMaxAutonomy = autonomie;
}

Path::Path(const Path & old_obj)
:mMaxAutonomy(old_obj.mMaxAutonomy)
, mStart(old_obj.mStart)
, mEnd(old_obj.mEnd)
, mDistance(old_obj.mDistance)
, mIsCompleted(old_obj.mIsCompleted)
{
	for each (auto var in old_obj.mSequenceAction)
	{
		mSequenceAction.push_back(var->clone());
	}
}

void Path::removeActionThatStartAfter(float fTimeLimit)
{

	auto eliminateAfter = mSequenceAction.end();
	for (auto it = mSequenceAction.begin(); it != mSequenceAction.end();++it)
	{
		if ((*it)->getStartTime() > fTimeLimit)
		{
			eliminateAfter = it;
			break;
		}
	}
	if (eliminateAfter != mSequenceAction.end())
	{
		mSequenceAction.erase(eliminateAfter, mSequenceAction.end());
		mIsCompleted = false;
		recomputeDistance();
	}
}

Path::Path()
{
}


Path::~Path()
{

	for each (auto var in mSequenceAction)
	{
		delete &(*var);
	}
}

Path & Path::operator=(const Path & old_obj)
{
	mMaxAutonomy = old_obj.mMaxAutonomy;
	mStart = old_obj.mStart;
	mEnd = old_obj.mEnd;
	mDistance = old_obj.mDistance;
	for each (auto var in mSequenceAction)
	{
		delete &(*var);
	}
	mSequenceAction.clear();
	for each (auto var in old_obj.mSequenceAction)
	{
		mSequenceAction.push_back(var->clone());
	}
	return *this;
}

string Path::toString()
{
	std::ostringstream ss;
	int roadSegmentCounter = 0;
	//ss << setw(20) << left << "Start point:" << mStart << endl;
	//ss << setw(20) << left << "End point:" << mEnd << endl;
	for each (auto var in mSequenceAction)
	{
		//Ignore road as it is too cumbersome
		if (var->getType() == Action::eRoad)
		{
			roadSegmentCounter++;
		}
		else
		{
			if (roadSegmentCounter > 0)
			{
				ss << setw(12) << left << Action::enumAssociation[Action::eRoad] << roadSegmentCounter << " segments" << endl;
				//ss << "..." << endl;
				roadSegmentCounter = 0;
			}
			ss << var->toString();
		}
	}

	if (roadSegmentCounter > 0)
	{
		ss << setw(12) << left << Action::enumAssociation[Action::eRoad] << roadSegmentCounter << " segments" << endl;
		roadSegmentCounter = 0;
	}
	return ss.str();
}

string Path::toStringInteractive()
{
	std::ostringstream ss;
	std::ostringstream st;
	ss << std::setprecision(8);
	for (std::list<Action*>::iterator iter = mSequenceAction.begin(); iter != mSequenceAction.end(); ++iter)
	{
		if ((*iter)->getType() == Action::eRoad)
		{
			ss << (*iter)->mPosition << '\t';
		}
		else
		{
			st << (*iter)->getName() << '\t';
		}

	}

	ss << endl;
	ss << st.str();

	return ss.str();
}

float Path::getDuration() const
{
	float retVal = INFINITY;
	if (!mSequenceAction.empty())
	{
		retVal = mSequenceAction.back()->getStartTime() + mSequenceAction.back()->getDuration() - mSequenceAction.front()->mStartTime;
	}
	return retVal;
}
