#include "SystemSolverImmediate.h"

#include "ScenarioData.h"
#include "SystemSolution.h"
#include "IVehicleSolver.h"
#include "IObjectiveFunction.h"
#include "VehicleSolution.h"
#include "Vehicle.h"
#include "ElectricCircuit.h"
#include "PerformanceAnalysis.h"
#include <iostream>

//Strong dependance
#include "VehicleSolverIgnoreStationOptimal.h"


SystemSolverImmediate::SystemSolverImmediate(IObjectiveFunction* inObjectiveFunction, IVehicleSolver* inVehicleSolver, ElectricCircuit* inElectricCircuit)
:SystemSolver(inObjectiveFunction,inVehicleSolver,nullptr,inElectricCircuit)
{
	mSystemSolution->mDescription = "Generic system solver.";
}


SystemSolverImmediate::~SystemSolverImmediate()
{
}

void SystemSolverImmediate::SolveSystem()
{
	if (!mIsActivated)
	{
		return;
	}
}

void SystemSolverImmediate::AddVehicleInternal(Vehicle * newVehicle)
{
	mPerformanceAnalysis->StartExperiment();

	//Optionnal data handling
	if (mSolverData->mObjOptionnalPartialSolution != nullptr)
	{
		mSystemSolution = mSolverData->mObjOptionnalPartialSolution;
	}
	VehicleSolverIgnoreStationOptimal* pSolverIgnoreStation = dynamic_cast<VehicleSolverIgnoreStationOptimal*>(mVehicleSolver);
	if (pSolverIgnoreStation != nullptr)
	{
		pSolverIgnoreStation->SetTimeHorizon(mSolverData->mfOptTimeHorizon);
	}
	//Init when we solve first vehicle
	if (mSolverData->mVecVehicle.size() == 0)
	{
		mSystemSolution->mSystemObjectiveFunctionValue = 0.0f;
	}

	SystemSolver::AddVehicleInternal(newVehicle);
	auto itSolutionVehicle = mSystemSolution->mMapVehicleSolution.find(newVehicle);
	VehicleSolution* vehicleSolution = nullptr;
	if (itSolutionVehicle != mSystemSolution->mMapVehicleSolution.end())
	{
		vehicleSolution = mSystemSolution->mMapVehicleSolution.at(newVehicle);
	}
	else
	{
		vehicleSolution = new VehicleSolution(newVehicle);
	}

	mVehicleSolver->SolveFor(newVehicle, vehicleSolution);

	if (vehicleSolution->isValid())
	{
		mSystemSolution->AddSolution(vehicleSolution);
		mSystemSolution->mSystemObjectiveFunctionValue += vehicleSolution->mCost;
	}
	else
	{
		delete vehicleSolution;
		std::cout << "No solution for " <<std::endl << *newVehicle << endl;
		std::cout << "Solution was deleted and ignored" << endl;
	}

	mPerformanceAnalysis->PauseExperiment(GetCurrentSystemSolution()->mSystemObjectiveFunctionValue);
}
