#include "SystemSolverSweepWindow.h"

#include "SystemSolver.h"
#include "ISystemSolver.h"
#include "ElectricCircuit.h"
#include "ScenarioData.h"
#include "SystemSolution.h"
#include "VehicleSolution.h"
#include "IVehicleSolver.h"
#include "VehicleSolverIgnoreStationOptimal.h"
#include "PerformanceAnalysis.h"
#include "Vehicle.h"
#include <assert.h>
#include <iostream>
#include <algorithm>




SystemSolverSweepWindow::SystemSolverSweepWindow(IObjectiveFunction* inObjectiveFunction, IVehicleSolver* inVehicleSolver, ElectricCircuit* inElectricCircuit, float infWindowWidth, float infResolveStep)
	:SystemSolver(inObjectiveFunction, inVehicleSolver,nullptr, inElectricCircuit)
	,mfWindowWidth(infWindowWidth)
	,mfResolveStep(infResolveStep)
{
}


SystemSolverSweepWindow::~SystemSolverSweepWindow()
{

}

void SystemSolverSweepWindow::SolveSystem()
{
	if (!mIsActivated)
	{
		return;
	}
	mPerformanceAnalysis->StartExperiment();

	for each (auto var in mSolverData->mVecVehicle)
	{
		mSystemSolution->AddSolution(new VehicleSolution(var));
	}

	//Strong dependency to VehicleSolverIgnoreStationOptimal
	VehicleSolverIgnoreStationOptimal* vs = dynamic_cast<VehicleSolverIgnoreStationOptimal*>(mVehicleSolver);
	assert(vs != nullptr);

	//Vehicle that leaves the sooner will dictate minimum time
	float fCurrentTime = mfMinimumDepartureTime;
	bool needToSolve = true;
	while (needToSolve)
	{
		//Sub-solver initialization
		needToSolve = false;
		mSystemSolution->mSystemObjectiveFunctionValue = 0.0f;
		float fNewTimeHorizon = fCurrentTime + mfWindowWidth;
		vs->SetTimeHorizon(fNewTimeHorizon);
		//Solving for each vehicle using the sub solver
		for each (auto var in mSystemSolution->mMapVehicleSolution)
		{
			VehicleSolution* newSolution = var.second;
			//Strip the solution from actions that started after time horizon
			bool wasChanged = newSolution->keepPartialSolution(fNewTimeHorizon);

			if (wasChanged || newSolution->actionCount() == 0)
			{
				needToSolve = true;
				mVehicleSolver->SolveFor(var.first, newSolution);
			}
			if (newSolution->isValid())
			{
				mSystemSolution->mSystemObjectiveFunctionValue += newSolution->mCost;
			}
			else
			{
				std::cout << "No solution for " << std::endl << *(var.first) << endl;
				std::cout << "Solution was deleted and ignored" << endl;
			}
		}
		fCurrentTime += mfResolveStep;
	}
	mPerformanceAnalysis->PauseExperiment(GetCurrentSystemSolution()->mSystemObjectiveFunctionValue);

}

void SystemSolverSweepWindow::AddVehicleInternal(Vehicle * newVehicle)
{
	mfMinimumDepartureTime = min(mfMinimumDepartureTime, newVehicle->GetDepartureTime());
	SystemSolver::AddVehicleInternal(newVehicle);
}