#include "VehicleSolution.h"

#include <string>
#include <sstream>
#include <iomanip>
#include <assert.h>
#include "Vehicle.h"
#include "Path.h"
#include "ElectricReservation.h"
#include "ActionTerminal.h"
#include "ElectricTerminal.h"

using namespace std;

VehicleSolution::VehicleSolution(Vehicle* inRefVehicle)
	:mRefVehicle(inRefVehicle)
{
	mPath = new Path(mRefVehicle->GetPointDeparture(), mRefVehicle->GetPointArrival(), mRefVehicle->GetAutonomie());
}

VehicleSolution::VehicleSolution(const VehicleSolution & old_obj)
:mRefVehicle(old_obj.mRefVehicle)
{
	this->mPath = new Path(*old_obj.mPath);
	this->mCost = old_obj.mCost;
}


VehicleSolution::~VehicleSolution()
{
	if (mPath != nullptr)
	{
		delete mPath;
	}
}

VehicleSolution & VehicleSolution::operator=(const VehicleSolution & other)
{
	if (mPath != nullptr)
	{
		delete mPath;
	}

	//TODO delete stuff so we avoid leaks
	this->mCost = other.mCost;
	this->mPath = new Path(*other.mPath);
	//Exception when copying another solution
	this->mRefVehicle = other.mRefVehicle;
	return *this;
}

VehicleSolution * VehicleSolution::clone() const
{
	return new VehicleSolution(*this);
}

string VehicleSolution::ComputeTimeRoundedSecondInteractive() const
{
	//TODO: not clean, should change visionneuse
	std::ostringstream ss;
	if (!IsImpossible())
		ss << (TimeUnit)round(mPath->getDuration()) << " s";
	else
		ss << cst_no_path << endl;

	return ss.str();
}

string VehicleSolution::toString() const
{
	std::ostringstream ss;
	ss << setw(20) << left << "Cost:" << mCost << endl;
	ss << setw(20) << left << "Distance:" << mPath->mDistance << endl;
	ss << setw(20) << left << "Duration: " << ComputeTimeRoundedSecondInteractive() << endl <<"---PATH---"<< endl << mPath->toString();
	return ss.str();
}

string VehicleSolution::toStringInteractive() const
{
	std::ostringstream ss;
	string duration = ComputeTimeRoundedSecondInteractive();
	ss << duration << endl;
	if (duration.compare(cst_no_path) != 0)
	{
		//Did we have to wait at any station? If so, compute if an alternate path is advantageous by ignoring the station we needed to wait
		for each (auto var in mPath->mSequenceAction)
		{
			if (var->getType() == Action::eRoad)
			{
				ss << var->getPosition() << "   ";
			}
		}
		ss << endl;
		for each (auto var in mPath->mSequenceAction)
		{
			if (var->getType() == Action::eTerminal)
			{
				ActionTerminal* acTerminal = dynamic_cast<ActionTerminal*>(var);
				ss << acTerminal->getTerminal()->GetName() << "   ";
			}
		}
		ss << endl;
	}
	else
	{
		ss << endl << endl;
	}

	return ss.str();
}

void VehicleSolution::CommitReservation(float fIgnoreAfterTime)
{
	//Did we have to wait at any station? If so, compute if an alternate path is advantageous by ignoring the station we needed to wait
	for each (auto var in mPath->mSequenceAction)
	{
		//Do not commit anything beyond time horizon
		if (var->getStartTime() > fIgnoreAfterTime)
			break;

		if (var->getType() == Action::eTerminal)
		{
			ActionTerminal* varTerminal = dynamic_cast<ActionTerminal*>(var);
			ElectricReservation* newReservation = new ElectricReservation(varTerminal->getTimeRange(), mRefVehicle);
			bool success = varTerminal->getTerminal()->TryReserve(newReservation);
			varTerminal->setCommitedReservation(newReservation);
			assert(success);
		}
	}
}

void VehicleSolution::RevertReservation(float fIgnoreBeforeTime)
{
	//Did we have to wait at any station? If so, compute if an alternate path is advantageous by ignoring the station we needed to wait
	for each (auto var in mPath->mSequenceAction)
	{
		if (var->getStartTime() <= fIgnoreBeforeTime)
		{
			continue;
		}
		if (var->getType() == Action::eTerminal)
		{
			ActionTerminal* varTerminal = dynamic_cast<ActionTerminal*>(var);
			bool success = varTerminal->getTerminal()->TryFree(varTerminal->getCommitedReservation());
			varTerminal->setCommitedReservation(nullptr);
			assert(success);
		}
	}
}

//Return true if we stripped some action
bool VehicleSolution::keepPartialSolution(float fLimitTime)
{
	size_t sizeBefore = mPath->mSequenceAction.size();
	mPath->removeActionThatStartAfter(fLimitTime);
	size_t sizeAfter = mPath->mSequenceAction.size();
	bool wasChanged = sizeAfter != sizeBefore;
	if (wasChanged)
	{
		mCost = INFINITY;
	}
	return wasChanged;
}

bool VehicleSolution::isValid() const
{
	//bool isValid = false;
	//if (!mPath->mSequenceAction.empty())
	//{
	//	Point startPath = mPath->mSequenceAction[0]->getPosition();
	//	Point endPath = mPath->mSequenceAction.back()->getPosition();
	//	isValid = startPath == mPath->mStart && endPath == mPath->mEnd;
	//}
	//return isValid;

	//TODO
	return true;
}

bool VehicleSolution::IsImpossible() const { return mPath->mDistance > 1e100; }

size_t VehicleSolution::actionCount() const
{
	return mPath->mSequenceAction.size();
}

ostream & operator<<(ostream & os, const VehicleSolution & dt)
{
	os << dt.toString();
	return os;
}
