#include "VehicleSolverIgnoreStationOptimal.h"

#include <queue>
#include <unordered_set>
#include "srcMap/carte.h"
#include "VehicleSolution.h"
#include "ElectricCircuit.h"
#include "RequestCarte.h"
#include "Path.h"
#include "ActionWait.h"
#include "IObjectiveFunction.h"

VehicleSolverIgnoreStationOptimal::VehicleSolverIgnoreStationOptimal(Carte const *const inCarte, ElectricCircuit * inRefElectricCircuit, IObjectiveFunction* inObjectiveFunction)
	:IVehicleSolver(inObjectiveFunction, inRefElectricCircuit)
	,mRefCarte(inCarte)
	,mRefObjectiveFunction(inObjectiveFunction)
{
}


VehicleSolverIgnoreStationOptimal::~VehicleSolverIgnoreStationOptimal()
{
}

void VehicleSolverIgnoreStationOptimal::SolveFor(Vehicle * inVehicleToSolve, VehicleSolution * out_RefSolution)
{
	//Sentinel
	bool bBestSolutionFound = false;

	//Request base
	RequestCarte request;
	request.mRefElectricCircuit = mRefElectricCircuit;
	request.mRefVehicleRequested = inVehicleToSolve;
	request.bIncludeOtherVehiclePresence = true;
	request.fIncludeOtherVehicleMaxTimeHorizon = mfTimeHorizon;

	//Structure to try alternative trajectories
	queue<ElectricTerminal*> qElectricTerminalToCheck;
	unordered_set<ElectricTerminal*> setElectricTerminalChecked;
	ElectricTerminal* currentlyIgnored = nullptr;
	VehicleSolution initialSolution(*out_RefSolution);

	while (!bBestSolutionFound)
	{
		//Compute new solution
		//Copy entire solution first
		VehicleSolution tmpSolution(initialSolution);
		(float)mRefCarte->calculerTrajet(*tmpSolution.mPath, request);
		tmpSolution.mCost = mRefObjectiveFunction->ComputeObjectiveFunction(&tmpSolution);

		//Did we found a better solution?
		if (tmpSolution.mCost < out_RefSolution->mCost)
		{
			//Yes! Copy it into the output solution
			*out_RefSolution = tmpSolution;
			//In order to keep it in the ignored station list, set null pointer to this
			currentlyIgnored = nullptr;
		}

		//Did we have to wait at any station? If so, compute if an alternate path is advantageous by ignoring the station we needed to wait
		//TODO: try to shuffle the wait at different stations
		for each (auto var in tmpSolution.mPath->mSequenceAction)
		{
			if (var->getType() == Action::eWait)
			{
				ActionWait* varWait = dynamic_cast<ActionWait*>(var);
				if (setElectricTerminalChecked.find(varWait->getTerminal()) == setElectricTerminalChecked.end())
				{
					qElectricTerminalToCheck.push(varWait->getTerminal());
				}
			}
		}

		bBestSolutionFound = qElectricTerminalToCheck.empty();
		//Prepare next solution search when we still stations to check up
		if (!bBestSolutionFound)
		{
			//Ignored station was not yielding better solution so we remove it
			if (currentlyIgnored != nullptr)
			{
				request.mOptionnal_StationToIgnore.erase(currentlyIgnored);
			}
			//Add a new station to ignore
			currentlyIgnored = qElectricTerminalToCheck.front();
			qElectricTerminalToCheck.pop();
			request.mOptionnal_StationToIgnore.insert(currentlyIgnored);
			//Add it to checked stations so we do not consider the same twice
			setElectricTerminalChecked.insert(currentlyIgnored);
		}
	}
	//cout << *out_RefSolution;
	//Commit the reservation to the electric circuit once we are done
	out_RefSolution->CommitReservation(mfTimeHorizon);
	//cout << *mRefElectricCircuit;
}
